import abc
import hashlib
from pathlib import Path

import pytest

from op_askpass.fingerprint_generator import (
    AbstractFingerprintGenerator,
    MD5FingerprintGenerator,
    SSHKeyGenFingerprintGenerator,
    KeyFingerprint,
)

HERE = Path(__file__).parent


class AbstractFingerprintGeneratorTests(abc.ABC):
    @pytest.fixture
    def test_key_file_path(self, tmp_path: Path) -> Path:
        file_path = tmp_path / "yada.txt"
        file_path.write_bytes(b"xxx")
        return file_path

    def test_for_path_should_correctly_generate_fingerprint(
        self, test_key_file_path: Path
    ) -> None:
        generator = self.fingerprint_generator_factory()

        assert isinstance(generator.for_path(test_key_file_path), KeyFingerprint)

    @abc.abstractmethod
    def fingerprint_generator_factory(self) -> AbstractFingerprintGenerator:
        ...


class TestMD5FingerprintGenerator(AbstractFingerprintGeneratorTests):
    def fingerprint_generator_factory(self) -> AbstractFingerprintGenerator:
        return MD5FingerprintGenerator()

    def test_for_path_should_generate_md5_hash(self, tmp_path: Path) -> None:
        generator = self.fingerprint_generator_factory()
        file_path = tmp_path / "yada.txt"
        file_path.write_bytes(b"xxx")

        assert generator.for_path(file_path) == KeyFingerprint(
            length=32,
            hash="f561aaf6ef0bf14d4208bb46a4ccb3ad",
            comment="some-key",
            key_type="md5",
        )


class TestSSHKeyGenFingerprintGenerator(AbstractFingerprintGeneratorTests):
    @pytest.fixture
    def test_key_file_path(self) -> Path:
        return Path(HERE / "test_data" / "test_key.pub")

    def fingerprint_generator_factory(self) -> AbstractFingerprintGenerator:
        return SSHKeyGenFingerprintGenerator()

    def test_for_path_should_generate_correct_checksum(
        self, test_key_file_path: Path
    ) -> None:
        generator = self.fingerprint_generator_factory()

        assert generator.for_path(test_key_file_path) == KeyFingerprint.from_str(
            "2048 SHA256:IKJs9nCZuYQ2tND/G2kCM1/+ggJsWMnWn+iP7y4FXDc test@key (RSA)"
        )



from typing import List, Tuple

from op_askpass.key_store import AbstractKeyStore


def list_keys(key_store: AbstractKeyStore) -> List[Tuple[str, str]]:
    return [
        (
            f"{fingerprint.length} {fingerprint.hash} {fingerprint.comment} ({fingerprint.key_type})",
            key_entry.onepass_uid,
        )
        for fingerprint, key_entry in key_store.items()
    ]



from pathlib import Path

from op_askpass.fingerprint_generator import KeyFingerprint
from op_askpass.key_store import MemoryKeyStore
from op_askpass.operations.list_keys import list_keys


def test_list_keys_should_return_list_of_added_keys(tmp_path: Path) -> None:
    key_store = MemoryKeyStore()
    fingerprint = KeyFingerprint(
        length=10, hash="some-fingerprint", comment="test", key_type="md5"
    )
    uid = "some-uid"
    path = Path("x")
    key_store.add_fingerprint(fingerprint=fingerprint, onepass_uid=uid, key_path=path)

    keys = list_keys(key_store=key_store)

    assert keys == [(fingerprint.to_str(), uid)]



from pathlib import Path

from op_askpass.fingerprint_generator import MD5FingerprintGenerator
from op_askpass.key_store import MemoryKeyStore, KeyEntry
from op_askpass.operations.add_key import add_key_from_path


def test_add_key_should_add_key_from_path(tmp_path: Path) -> None:
    key_path = tmp_path / "yada.txt"
    key_path.write_text("xxx")
    key_store = MemoryKeyStore()
    fingerprint_generator = MD5FingerprintGenerator()
    uid = "some-uid"

    add_key_from_path(
        path=key_path,
        onepass_uid=uid,
        fingerprint_generator=fingerprint_generator,
        key_store=key_store,
    )

    assert key_store.get_key_entry(
        fingerprint=fingerprint_generator.for_path(key_path)
    ) == KeyEntry(onepass_uid=uid, key_path=key_path)



import abc
from pathlib import Path

from op_askpass.key_loader import AbstractKeyLoader, MemoryKeyLoader


class AbstractKeyLoaderTests(abc.ABC):
    @abc.abstractmethod
    def key_loader_factory(self) -> AbstractKeyLoader:
        ...

    def test_list_loaded_keys_should_return_empty_list_when_empty(self) -> None:
        loader = self.key_loader_factory()

        actual = loader.list_loaded_keys()

        assert actual == []

    def test_load_key_should_correctly_load_given_key(self) -> None:
        loader = self.key_loader_factory()
        fingerprint_generator = loader.get_fingerprint_generator()
        key_path = Path(__file__).parent / "test_data" / "test_key"

        loader.load_key(
            key_path=key_path,
            op_domain="some-domain",
            op_uid="some-uid",
            op_session_key="some-key",
        )

        assert loader.list_loaded_keys() == [fingerprint_generator.for_path(key_path)]


class TestMemoryKeyLoader(AbstractKeyLoaderTests):
    def key_loader_factory(self) -> AbstractKeyLoader:
        return MemoryKeyLoader()



import os

from op_askpass.configuration import get_configuration_directory
from op_askpass.one_password.client import OPClient, find_executable_in_env


def main() -> None:
    item_name = os.environ["OP_ASKPASS_ITEM_NAME"]
    op_client = OPClient(
        executable_path=find_executable_in_env(os.environ)
        or get_configuration_directory() / "op"
    )
    print(op_client.get_password(item_name=item_name))



import abc
from logging import Logger
from pathlib import Path

import pytest

from op_askpass.one_password.file_verifier import (
    AbstractFileVerifier,
    MD5FileVerifier,
    VerificationError,
    GPGFileVerifier,
)


class AbstractBinaryVerifierTests(abc.ABC):
    @abc.abstractmethod
    def get_signature_file_name(self) -> str:
        ...

    @abc.abstractmethod
    def verifier_factory(self, logger: Logger) -> AbstractFileVerifier:
        ...

    def test_verify_file_signature_should_correctly_verify_signature(
        self, null_logger: Logger
    ) -> None:
        verifier: AbstractFileVerifier = self.verifier_factory(logger=null_logger)
        test_data_path = Path(__file__).parent.parent / "test_data"
        key_path = test_data_path / "test_key"
        signature_path = test_data_path / self.get_signature_file_name()

        verifier.verify_file_signature(
            file_path=key_path, signature=signature_path.read_bytes().strip()
        )

    def test_verify_file_signature_should_raise_verification_error_on_incorrect_signature(
        self, null_logger: Logger
    ) -> None:
        verifier: AbstractFileVerifier = self.verifier_factory(logger=null_logger)
        test_data_path = Path(__file__).parent.parent / "test_data"
        key_path = test_data_path / "test_key"

        with pytest.raises(VerificationError, match=r"Wrong signature\."):
            verifier.verify_file_signature(
                file_path=key_path, signature=key_path.read_bytes().strip()
            )


class TestMD5BinaryVerifier(AbstractBinaryVerifierTests):
    def verifier_factory(self, logger: Logger) -> AbstractFileVerifier:
        return MD5FileVerifier()

    def get_signature_file_name(self) -> str:
        return "test_key.md5"


class TestGPGBinaryVerifier(AbstractBinaryVerifierTests):
    def get_signature_file_name(self) -> str:
        return "test_key.sig"

    def verifier_factory(self, logger: Logger) -> AbstractFileVerifier:
        return GPGFileVerifier(logger=logger)



from pathlib import Path

from op_askpass.fingerprint_generator import MD5FingerprintGenerator
from op_askpass.key_store import MemoryKeyStore
from op_askpass.operations.delete_key import delete_key_from_path


def test_delete_key_from_path_should_remove_the_key(tmp_path: Path) -> None:
    key_path = tmp_path / "yada.txt"
    key_path.write_text("xxx")
    key_store = MemoryKeyStore()
    fingerprint_generator = MD5FingerprintGenerator()
    uid = "some-uid"
    fingerprint = fingerprint_generator.for_path(key_path)
    key_store.add_fingerprint(
        fingerprint=fingerprint, onepass_uid=uid, key_path=key_path
    )

    delete_key_from_path(
        path=key_path, fingerprint_generator=fingerprint_generator, key_store=key_store
    )

    assert key_store.items() == []



from pathlib import Path

from op_askpass.fingerprint_generator import AbstractFingerprintGenerator
from op_askpass.key_store import AbstractKeyStore


def add_key_from_path(
    path: Path,
    onepass_uid: str,
    fingerprint_generator: AbstractFingerprintGenerator,
    key_store: AbstractKeyStore,
) -> None:
    fingerprint = fingerprint_generator.for_path(path=path)
    key_store.add_fingerprint(
        fingerprint=fingerprint, onepass_uid=onepass_uid, key_path=path.absolute()
    )



from pathlib import Path

import pytest

from op_askpass.one_password.client import find_executable_in_directories


@pytest.mark.integration
def test_find_executable_in_dirs(tmp_path: Path) -> None:
    empty_dir = tmp_path / "empty"
    empty_dir.mkdir()
    client_dir = tmp_path / "contains"
    client_dir.mkdir()
    (client_dir / "op").write_bytes(b"test")
    without_client_dir = tmp_path / "no_client"
    without_client_dir.mkdir()
    (without_client_dir / "something.txt").write_text("test")

    assert (
        find_executable_in_directories([empty_dir, client_dir, without_client_dir])
        == client_dir / "op"
    )



from op_askpass.operations.add_key import add_key_from_path
from op_askpass.operations.delete_key import delete_key_from_path
from op_askpass.operations.list_keys import list_keys
from op_askpass.operations.op_client import get_password_from_op
from op_askpass.operations.login import login_to_op
from op_askpass.operations.setup_client import setup_op_client



import abc
import json
from dataclasses import dataclass
from pathlib import Path
from typing import Dict, List, Tuple

from op_askpass.configuration import get_configuration_directory
from op_askpass.fingerprint_generator import KeyFingerprint


@dataclass
class KeyEntry:
    key_path: Path
    onepass_uid: str


class AbstractKeyStore(abc.ABC):
    @abc.abstractmethod
    def add_fingerprint(
        self, fingerprint: KeyFingerprint, onepass_uid: str, key_path: Path
    ) -> None:
        ...

    @abc.abstractmethod
    def delete_fingerprint(self, fingerprint: KeyFingerprint) -> None:
        ...

    @abc.abstractmethod
    def get_key_entry(self, fingerprint: KeyFingerprint) -> KeyEntry:
        ...

    @abc.abstractmethod
    def items(self) -> List[Tuple[KeyFingerprint, KeyEntry]]:
        ...


class MemoryKeyStore(AbstractKeyStore):
    def delete_fingerprint(self, fingerprint: KeyFingerprint) -> None:
        self.__store.pop(fingerprint, None)

    def items(self) -> List[Tuple[KeyFingerprint, KeyEntry]]:
        return list(self.__store.items())

    def get_key_entry(self, fingerprint: KeyFingerprint) -> KeyEntry:
        return self.__store[fingerprint]

    def __init__(self) -> None:
        self.__store: Dict[KeyFingerprint, KeyEntry] = {}

    def add_fingerprint(
        self, fingerprint: KeyFingerprint, onepass_uid: str, key_path: Path
    ) -> None:
        self.__store[fingerprint] = KeyEntry(onepass_uid=onepass_uid, key_path=key_path)


class FileKeyStore(AbstractKeyStore):
    def delete_fingerprint(self, fingerprint: KeyFingerprint) -> None:
        contents = self.__read_contents(self.__file_path)
        contents.pop(fingerprint, None)
        self.__save_contents(file_path=self.__file_path, contents=contents)

    def items(self) -> List[Tuple[KeyFingerprint, KeyEntry]]:
        return list(self.__read_contents(self.__file_path).items())

    def __init__(self, file_path: Path) -> None:
        self.__file_path = file_path

    @staticmethod
    def __read_contents(file_path: Path) -> Dict[KeyFingerprint, KeyEntry]:
        try:
            data = json.loads(file_path.read_text(encoding="utf-8"))
            return {
                KeyFingerprint.from_str(k): KeyEntry(
                    onepass_uid=v["onepass_uid"], key_path=Path(v["key_path"])
                )
                for k, v in data.items()
            }

        except IOError:
            return {}

    @staticmethod
    def __save_contents(
        file_path: Path, contents: Dict[KeyFingerprint, KeyEntry]
    ) -> None:
        file_path.write_text(
            json.dumps(
                {
                    k.to_str(): {
                        "onepass_uid": v.onepass_uid,
                        "key_path": str(v.key_path),
                    }
                    for k, v in contents.items()
                }
            ),
            encoding="utf-8",
        )

    def add_fingerprint(
        self, fingerprint: KeyFingerprint, onepass_uid: str, key_path: Path
    ) -> None:
        contents = self.__read_contents(self.__file_path)
        contents[fingerprint] = KeyEntry(onepass_uid=onepass_uid, key_path=key_path)
        self.__save_contents(file_path=self.__file_path, contents=contents)

    def get_key_entry(self, fingerprint: KeyFingerprint) -> KeyEntry:
        return self.__read_contents(self.__file_path)[fingerprint]


def get_default_key_store() -> AbstractKeyStore:
    return FileKeyStore(file_path=get_configuration_directory() / "op-askpass.json")



from op_askpass.key_loader import AbstractKeyLoader
from op_askpass.key_store import AbstractKeyStore
from op_askpass.one_password.client import AbstractOPClient


def login_to_op(
    op_client: AbstractOPClient,
    op_domain: str,
    key_store: AbstractKeyStore,
    key_loader: AbstractKeyLoader,
    skip_existing: bool = True,
) -> None:
    session_key = op_client.login_to_domain(domain=op_domain)
    loaded_fingerprints = set(key_loader.list_loaded_keys())
    for fingerprint, key_entry in key_store.items():
        if fingerprint in loaded_fingerprints and skip_existing:
            print(f"Skipping key {key_entry.key_path}. Already loaded.")
            continue

        key_loader.load_key(
            key_path=key_entry.key_path,
            op_domain=op_domain,
            op_session_key=session_key,
            op_uid=key_entry.onepass_uid,
        )



import shutil
import tempfile
from logging import Logger
from pathlib import Path
from typing import Optional

from op_askpass.one_password.client import AbstractOPClient, OPClient
from op_askpass.one_password.downloader import AbstractDownloader, HTTPDownloader
from op_askpass.one_password.file_verifier import AbstractFileVerifier


def setup_op_client(
    download_url: str,
    install_path: Path,
    op_domain: str,
    op_email: str,
    logger: Logger,
    verifier: Optional[AbstractFileVerifier] = None,
    downloader: AbstractDownloader = None,
    op_client: Optional[AbstractOPClient] = None,
) -> None:
    downloader = downloader or HTTPDownloader(logger=logger)

    with tempfile.TemporaryDirectory() as tmpdir:
        tmpdir = Path(str(tmpdir))
        download_dir = tmpdir / "downloaded_client"
        download_dir.mkdir()
        downloader.download_from_url(url=download_url, target_dir=download_dir)

        if verifier:
            verifier.verify_file_signature(
                file_path=download_dir / "op",
                signature=(download_dir / "op.sig").read_bytes(),
            )

        shutil.copy(src=str(download_dir / "op"), dst=str(install_path / "op"))

    op_client = op_client or OPClient(executable_path=install_path / "op")
    op_client.sign_in(domain=op_domain, email=op_email)



import abc
import os
import tempfile
import zipfile
from logging import Logger
from pathlib import Path
from typing import Dict, Optional

import requests


class AbstractDownloader(abc.ABC):
    @abc.abstractmethod
    def download_from_url(self, url: str, target_dir: Path) -> None:
        ...


class SimpleDownloader(AbstractDownloader):
    def download_from_url(self, url: str, target_dir: Path) -> None:
        for file_path, contents in self.__files.items():
            path_in_target = target_dir / file_path
            path_in_target.parent.mkdir(exist_ok=True)
            (target_dir / file_path).write_bytes(contents)

    def __init__(self, files: Dict[str, bytes]) -> None:
        self.__files = files.copy()


class ZipFileWithPermissions(zipfile.ZipFile):
    def _extract_member(
        self,
        member: Optional[zipfile.ZipInfo],
        targetpath: str,
        pwd: Optional[str] = None,
    ) -> str:
        if not isinstance(member, zipfile.ZipInfo):
            member = self.getinfo(member)

        targetpath = super()._extract_member(member, targetpath, pwd)

        attr = member.external_attr >> 16
        if attr != 0:
            os.chmod(targetpath, attr)
        return targetpath


class HTTPDownloader(AbstractDownloader):
    def __init__(self, logger: Logger) -> None:
        self.__logger = logger

    def download_from_url(self, url: str, target_dir: Path) -> None:
        self.__logger.info("Fetching: %s", url)

        with tempfile.TemporaryDirectory() as tmpdir:
            tmpdir = Path(tmpdir)
            download_path = tmpdir / "downloaded.zip"
            with open(download_path, "wb") as downloaded_file, requests.get(
                url=url, stream=True
            ) as stream:
                for chunk in stream:
                    downloaded_file.write(chunk)

            with ZipFileWithPermissions(download_path) as archive:
                archive.extractall(path=target_dir)



from logging import Logger
from pathlib import Path
from typing import Any, Callable

import pytest

from op_askpass.one_password.client import AbstractOPClient
from op_askpass.one_password.downloader import HTTPDownloader
from op_askpass.one_password.file_verifier import GPGFileVerifier
from op_askpass.operations import setup_op_client


class MockOPClient(AbstractOPClient):
    def login_to_domain(self, domain: str) -> str:
        pass

    def get_password(self, item_name: str) -> str:
        pass

    def sign_in(self, domain: str, email: str) -> None:
        pass


@pytest.mark.integration
def test_with_mocked_client_should_correctly_create_binary(
    tmp_path: Path, when: Callable[[Any], Any], null_logger: Logger
) -> None:
    op_client = MockOPClient()
    when(op_client).sign_in(domain="some-domain", email="some-email")

    setup_op_client(
        download_url="https://cache.agilebits.com/dist/1P/op/pkg/v0.5.7/op_linux_amd64_v0.5.7.zip",
        install_path=tmp_path,
        op_domain="some-domain",
        op_email="some-email",
        logger=null_logger,
        verifier=GPGFileVerifier(logger=null_logger),
        downloader=HTTPDownloader(logger=null_logger),
        op_client=op_client,
    )

    executable_path = tmp_path / "op"
    assert oct(executable_path.stat().st_mode & 0o777) == oct(0o755)



import abc
import hashlib
import subprocess
import tempfile
from logging import Logger
from pathlib import Path


class VerificationError(Exception):
    pass


class AbstractFileVerifier(abc.ABC):
    @abc.abstractmethod
    def verify_file_signature(self, file_path: Path, signature: bytes) -> None:
        ...


class MD5FileVerifier(AbstractFileVerifier):
    def verify_file_signature(self, file_path: Path, signature: bytes) -> None:
        md5 = hashlib.md5()
        with file_path.open(mode="rb") as f:
            for chunk in f:
                md5.update(chunk)

        if md5.hexdigest() != signature.decode("utf-8"):
            raise VerificationError("Wrong signature.")


class GPGFileVerifier(AbstractFileVerifier):
    def __init__(self, logger: Logger) -> None:
        self.__logger = logger

    def verify_file_signature(self, file_path: Path, signature: bytes) -> None:
        self.__logger.info("Verifying GPG signature: %s", file_path)

        with tempfile.NamedTemporaryFile() as tmp:
            tmp_path = Path(tmp.name)
            tmp_path.write_bytes(signature)

            try:
                subprocess.check_call(
                    ["gpg", "--verify", str(tmp_path), str(file_path)]
                )

            except subprocess.CalledProcessError as e:
                raise VerificationError("Wrong signature.") from e



import abc
import uuid
from pathlib import Path

import pytest

from op_askpass.fingerprint_generator import KeyFingerprint
from op_askpass.key_store import (
    AbstractKeyStore,
    MemoryKeyStore,
    FileKeyStore,
    KeyEntry,
)


class AbstractKeyStoreTests(abc.ABC):
    def test_add_key_fingerprint_should_correctly_add_fingerprint(self) -> None:
        key_store: AbstractKeyStore = self.key_store_factory()
        onepass_uid = "XXXXXX"
        fingerprint = KeyFingerprint(
            length=10, hash="SSSSSSSSS", comment="x", key_type="some"
        )
        path = Path("x")
        expected_entry = KeyEntry(onepass_uid=onepass_uid, key_path=path)

        key_store.add_fingerprint(
            fingerprint=fingerprint, onepass_uid=onepass_uid, key_path=path
        )

        assert key_store.get_key_entry(fingerprint=fingerprint) == expected_entry
        assert key_store.items() == [(fingerprint, expected_entry)]

    def test_delete_key_fingerprint_should_correctly_remove_a_fingerprint(self) -> None:
        key_store: AbstractKeyStore = self.key_store_factory()
        onepass_uid = "XXXXXX"
        fingerprint = KeyFingerprint(
            length=10, hash="SSSSSSSSS", comment="x", key_type="some"
        )
        path = Path("x")
        key_store.add_fingerprint(
            fingerprint=fingerprint, onepass_uid=onepass_uid, key_path=path
        )

        key_store.delete_fingerprint(fingerprint=fingerprint)

        with pytest.raises(KeyError, match=fingerprint.hash):
            key_store.get_key_entry(fingerprint=fingerprint)
        assert key_store.items() == []

    def test_get_onepass_uid_should_raise_key_error_on_missing_fingerprint(
        self
    ) -> None:
        key_store: AbstractKeyStore = self.key_store_factory()

        with pytest.raises(KeyError, match="not-existing"):
            key_store.get_key_entry(
                fingerprint=KeyFingerprint(
                    length=10, hash="not-existing", comment="x", key_type="some"
                )
            )

    @abc.abstractmethod
    def key_store_factory(self) -> AbstractKeyStore:
        ...


class TestMemoryKeyStore(AbstractKeyStoreTests):
    def key_store_factory(self) -> AbstractKeyStore:
        return MemoryKeyStore()


class TestFileKeyStore(AbstractKeyStoreTests):
    @pytest.fixture(autouse=True)
    def test_dir_factory(self, tmpdir) -> None:
        self.__tmpdir = tmpdir

    def key_store_factory(self) -> AbstractKeyStore:
        return FileKeyStore(file_path=self.__tmpdir / str(uuid.uuid4()))



import abc
import json
import subprocess
from pathlib import Path
from typing import List, Optional, Dict


class AbstractOPClient(abc.ABC):
    @abc.abstractmethod
    def sign_in(self, domain: str, email: str) -> None:
        ...

    @abc.abstractmethod
    def login_to_domain(self, domain: str) -> str:
        ...

    @abc.abstractmethod
    def get_password(self, item_name: str) -> str:
        ...


class OPClient(AbstractOPClient):
    def get_password(self, item_name: str) -> str:
        output = (
            subprocess.check_output(
                [str(self.__executable_path), "get", "item", item_name]
            )
            .decode("utf-8")
            .strip()
        )
        return json.loads(output)["details"]["password"]

    def login_to_domain(self, domain: str) -> str:
        return (
            subprocess.check_output(
                [str(self.__executable_path), "signin", domain, "--output=raw"]
            )
            .decode("utf-8")
            .strip()
        )

    def __init__(self, executable_path: Path) -> None:
        self.__executable_path = executable_path

    def sign_in(self, domain: str, email: str) -> None:
        subprocess.check_call(
            [str(self.__executable_path), "signin", domain, email, "--output=raw"]
        )


def find_executable_in_directories(directories: List[Path]) -> Optional[Path]:
    for directory in directories:
        possible_path = directory / "op"
        if possible_path.is_file():
            return possible_path

    return None


def find_executable_in_env(env: Dict[str, str]) -> Optional[Path]:
    return find_executable_in_directories(
        [Path(p) for p in env.get("PATH", "").split(":")]
    )



import logging
import os
from logging import StreamHandler
from pathlib import Path
from typing import Optional

import click

import op_askpass.operations.login
import op_askpass.operations.setup_client
from op_askpass import operations
from op_askpass.configuration import get_configuration_directory
from op_askpass.fingerprint_generator import SSHKeyGenFingerprintGenerator
from op_askpass.key_loader import SSHKeyLoader
from op_askpass.key_store import get_default_key_store
from op_askpass.one_password.client import OPClient, find_executable_in_env
from op_askpass.one_password.file_verifier import GPGFileVerifier


def setup_logging(verbosity: int = 0) -> logging.Logger:
    logger = logging.getLogger(__name__)
    logger.addHandler(StreamHandler())
    logger.setLevel(logging.INFO if verbosity else logging.WARNING)
    return logger


@click.group()
def main() -> None:
    """Yada yada"""


@main.command()
def list_keys() -> None:
    """ Print already stored keys fingerprint and their 1Password uids. """
    keys = operations.list_keys(key_store=get_default_key_store())
    for fingerprint, onepass_uid in keys:
        print(fingerprint, "||", onepass_uid)


@main.command()
@click.argument("op_domain")
@click.argument("op_email")
@click.option(
    "--install-path", type=Path, help="Where the 1Password client should be installed."
)
@click.option(
    "--verify/--no-verify",
    default=True,
    help="Should the download 1Password client binary be verified.",
)
@click.option("--verbose", "-v", "verbosity", type=int, count=True)
def setup_op_client(
    op_domain: str,
    op_email: str,
    verify: bool,
    install_path: Optional[Path],
    verbosity: int,
) -> None:
    """ Download the 1Password API client and login to email in domain. """
    logger = setup_logging(verbosity=verbosity)

    op_askpass.operations.setup_client.setup_op_client(
        download_url="https://cache.agilebits.com/dist/1P/op/pkg/v0.10.0/op_linux_amd64_v0.10.0.zip",
        install_path=install_path or get_configuration_directory(),
        op_domain=op_domain,
        op_email=op_email,
        logger=logger,
        verifier=GPGFileVerifier(logger=logger) if verify else None,
    )


@main.command()
@click.argument("op_domain")
@click.option(
    "--install-path", type=Path, help="Where the 1Password client was installed."
)
@click.option(
    "--skip-existing/--no-skip-existing",
    default=True,
    help="Skip keys already existing in the ssh agent.",
)
def login(
    op_domain: str, install_path: Optional[Path], skip_existing: bool = True
) -> None:
    if install_path:
        executable_path = install_path / "op"
    else:
        executable_path = (
            find_executable_in_env(os.environ) or get_configuration_directory() / "op"
        )

    op_askpass.operations.login.login_to_op(
        op_client=OPClient(executable_path=executable_path),
        op_domain=op_domain,
        key_store=get_default_key_store(),
        key_loader=SSHKeyLoader(),
        skip_existing=skip_existing,
    )


@main.command()
@click.argument("public_or_private_key", type=Path)
def delete_key(public_or_private_key: Path) -> None:
    """ Remove given key from prompt-less store. """
    operations.delete_key_from_path(
        path=public_or_private_key,
        fingerprint_generator=SSHKeyGenFingerprintGenerator(),
        key_store=get_default_key_store(),
    )


@main.command()
@click.argument("public_or_private_key", type=Path)
@click.argument("onepass_name_or_uid", type=str)
def add_key(public_or_private_key: Path, onepass_name_or_uid: str) -> None:
    """ Add given key with password stored in 1Password. The key will be available later on for prompt-less add. """
    operations.add_key_from_path(
        path=public_or_private_key,
        onepass_uid=onepass_name_or_uid,
        fingerprint_generator=SSHKeyGenFingerprintGenerator(),
        key_store=get_default_key_store(),
    )



import abc
import hashlib
import subprocess
from dataclasses import dataclass
from pathlib import Path
from typing import Any


@dataclass
class KeyFingerprint:
    length: int
    hash: str
    comment: str
    key_type: str

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, KeyFingerprint):
            return NotImplemented

        return (
            self.length == other.length
            and self.hash == other.hash
            and self.key_type == other.key_type
        )

    def __hash__(self) -> int:
        return hash(self.length) + hash(self.hash) + hash(self.key_type)

    @classmethod
    def from_str(cls, s: str) -> "KeyFingerprint":
        length, hash, comment, key_type = s.split()
        return KeyFingerprint(
            length=int(length),
            hash=hash,
            comment=comment,
            key_type=key_type.strip("(").strip(")"),
        )

    def to_str(self) -> str:
        return f"{self.length} {self.hash} {self.comment} ({self.key_type})"


class AbstractFingerprintGenerator(abc.ABC):
    @abc.abstractmethod
    def for_path(self, path: Path) -> KeyFingerprint:
        ...


class MD5FingerprintGenerator(AbstractFingerprintGenerator):
    def for_path(self, path: Path) -> KeyFingerprint:
        md5 = hashlib.md5()
        md5.update(path.read_bytes())
        hex = str(md5.hexdigest())
        return KeyFingerprint(
            length=len(hex), hash=hex, comment="some-key", key_type="md5"
        )


class SSHKeyGenFingerprintGenerator(AbstractFingerprintGenerator):
    def __init__(self, executable_name: str = "ssh-keygen") -> None:
        self.__executable_name = executable_name

    def for_path(self, path: Path) -> KeyFingerprint:
        output = subprocess.check_output(
            [self.__executable_name, "-l", "-f", path], encoding="utf-8"
        )
        return KeyFingerprint.from_str(output.strip())



import abc
from logging import Logger
from pathlib import Path
from stat import S_IEXEC, S_IRUSR, S_IWUSR, S_IRGRP, S_IROTH

import pytest

from op_askpass.one_password.downloader import (
    AbstractDownloader,
    SimpleDownloader,
    HTTPDownloader,
)

TEST_COMMIT_HASH = "027273ecb987e86958c19987a711c9bc0df83c64"


class AbstractDownloaderTests(abc.ABC):
    @abc.abstractmethod
    def downloader_factory(self, logger: Logger) -> AbstractDownloader:
        ...

    def test_download_from_url_should_download_to_given_path(
        self, tmp_path: Path, null_logger: Logger
    ) -> None:
        downloader = self.downloader_factory(logger=null_logger)

        downloader.download_from_url(
            url="https://cache.agilebits.com/dist/1P/op/pkg/v0.5.7/op_linux_amd64_v0.5.7.zip",
            target_dir=tmp_path,
        )

        assert (tmp_path / "op").exists()


class TestSimpleDownloader(AbstractDownloaderTests):
    def downloader_factory(self, logger: Logger) -> AbstractDownloader:
        return SimpleDownloader(files={"op": b"some content"})


@pytest.mark.integration
class TestHTTPDownloader(AbstractDownloaderTests):
    def downloader_factory(self, logger: Logger) -> AbstractDownloader:
        return HTTPDownloader(logger=logger)

    def test_download_from_url_should_download_to_given_path(
        self, tmp_path: Path, null_logger: Logger
    ) -> None:
        expected_mode = S_IEXEC | S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH

        super().test_download_from_url_should_download_to_given_path(
            tmp_path=tmp_path, null_logger=null_logger
        )

        assert (tmp_path / "op").stat().st_mode & expected_mode == expected_mode



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fh:
    requirements = fh.read().splitlines()

with open("dev_requirements.txt", "r") as fh:
    dev_requirements = fh.read().splitlines()

setup(
    name="op_askpass",
    version="0.0.12",
    author="Maciej Gol",
    author_email="1kroolik1@gmail.com",
    description="Add password-protected ssh keys promptless using 1Password.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/maciej.gol/op-askpass",
    packages=find_packages(),
    license="BSD-3-Clause",
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=requirements,
    extras_require={"dev": dev_requirements},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: Unix",
        "Environment :: Console",
        "Typing :: Typed",
    ],
    entry_points={
        "console_scripts": [
            "op-askpass=op_askpass.cli.main:main",
            "op-askpass-get-password=op_askpass.cli.get_password:main",
        ]
    },
)



import json
import subprocess
from pathlib import Path

__all__ = ["get_password_from_op"]


def get_password_from_op(executable: Path, item_name: str) -> str:
    output = (
        subprocess.check_output([str(executable), "get", "item", item_name])
        .decode("utf-8")
        .strip()
    )
    return json.loads(output)["details"]["password"]



import abc
import os
import subprocess
from pathlib import Path
from typing import List, Set

from op_askpass.fingerprint_generator import (
    AbstractFingerprintGenerator,
    MD5FingerprintGenerator,
    SSHKeyGenFingerprintGenerator,
    KeyFingerprint,
)


class AbstractKeyLoader(abc.ABC):
    @abc.abstractmethod
    def list_loaded_keys(self) -> List[KeyFingerprint]:
        ...

    @abc.abstractmethod
    def load_key(
        self, key_path: Path, op_domain: str, op_session_key: str, op_uid: str
    ) -> None:
        ...

    @abc.abstractmethod
    def get_fingerprint_generator(self) -> AbstractFingerprintGenerator:
        ...


class MemoryKeyLoader(AbstractKeyLoader):
    def get_fingerprint_generator(self) -> AbstractFingerprintGenerator:
        return MD5FingerprintGenerator()

    def __init__(self) -> None:
        self.__keys: Set[KeyFingerprint] = set()

    def load_key(
        self, key_path: Path, op_domain: str, op_session_key: str, op_uid: str
    ) -> None:
        self.__keys.add(self.get_fingerprint_generator().for_path(key_path))

    def list_loaded_keys(self) -> List[KeyFingerprint]:
        return list(self.__keys)


class SSHKeyLoader(AbstractKeyLoader):
    def get_fingerprint_generator(self) -> AbstractFingerprintGenerator:
        return SSHKeyGenFingerprintGenerator()

    def list_loaded_keys(self) -> List[KeyFingerprint]:
        output: str = subprocess.check_output(["ssh-add", "-l"], encoding="utf-8")
        return [KeyFingerprint.from_str(s) for s in output.splitlines()]

    def load_key(
        self, key_path: Path, op_domain: str, op_session_key: str, op_uid: str
    ) -> None:
        env = {**os.environ, f"OP_SESSION_{op_domain}": op_session_key}

        subprocess.check_call(
            ["ssh-add", str(key_path)],
            env={
                **env,
                "OP_ASKPASS_ITEM_NAME": op_uid,
                "SSH_ASKPASS": "op-askpass-get-password",
            },
            stdin=subprocess.DEVNULL,
        )



from pathlib import Path

from op_askpass.fingerprint_generator import AbstractFingerprintGenerator
from op_askpass.key_store import AbstractKeyStore


def delete_key_from_path(
    path: Path,
    fingerprint_generator: AbstractFingerprintGenerator,
    key_store: AbstractKeyStore,
) -> None:
    fingerprint = fingerprint_generator.for_path(path=path)
    key_store.delete_fingerprint(fingerprint=fingerprint)
